import 'dart:async';
import 'dart:io';
import 'dart:typed_data';
import 'dart:ui' as ui show Codec;
import 'package:flutter/foundation.dart';
import 'package:flutter/painting.dart';
import 'package:path/path.dart';
import '../flutter_mylibrary.dart';
import 'image_api.dart';
import 'package:path_provider/path_provider.dart';


class NetworkImageAsyncProvider extends ImageProvider<NetworkImageAsyncProvider> {
  /// Creates an object that fetches the image at the given URL.
  ///
  /// The arguments must not be null.
  const NetworkImageAsyncProvider(this.url, {this.scale = 1.0, this.headers, this.cache = true})
      : assert(url != null),
        assert(scale != null);

  /// The URL from which the image will be fetched.
  final String url;

  /// The scale to place in the [ImageInfo] object of the image.
  final double scale;

  /// The HTTP headers that will be used with [HttpClient.get] to fetch image from network.
  final Map<String, String> headers;

  ///whether cache image to local
  final bool cache;

  @override
  Future<NetworkImageAsyncProvider> obtainKey(ImageConfiguration configuration) {
    return SynchronousFuture<NetworkImageAsyncProvider>(this);
  }

  @override
  ImageStreamCompleter load(
      NetworkImageAsyncProvider key, DecoderCallback decode) {
    return MultiFrameImageStreamCompleter(
      codec: _loadAsync(key),
      scale: key.scale,
      informationCollector: () sync* {
        yield DiagnosticsProperty<ImageProvider>('Image provider', this);
        yield DiagnosticsProperty<NetworkImageAsyncProvider>('Image key', key);
      },
    );
  }

  static final HttpClient _httpClient = HttpClient();

  Future<ui.Codec> _loadAsync(NetworkImageAsyncProvider key) async {
    assert(key == this);
    final String md5Key = keyToMd5(key.url);
    Uint8List bytes;
    if (cache) {
      try {
        bytes = await _loadCache(
          key,
          md5Key,
        );
      } catch (e) {
        print(e);
      }
    }

    if (bytes == null) {
      try {
        bytes = await _loadNetwork(key);
      } catch (e) {
        print(e);
      }
    }

    //Failed to load
    if (bytes == null || bytes.length == 0) {
      //result = await ui.instantiateImageCodec(kTransparentImage);
      return Future<ui.Codec>.error(StateError('Failed to load $url.'));
    }

    return PaintingBinding.instance.instantiateImageCodec(bytes);
  }

  Future<Uint8List> _loadNetwork(NetworkImageAsyncProvider key) async{
    //解决不安全证书校验通不过的问题
    _httpClient.badCertificateCallback = (X509Certificate cert, String host, int port) {
      return true;
    };
    final Uint8List bytes = await ImageApi.image(key.url);
    if (bytes.lengthInBytes == 0)
      throw Exception('NetworkImage is an empty file');
    return bytes;
  }

  ///get the image from cache folder.
  Future<Uint8List> _loadCache(
      NetworkImageAsyncProvider key,
      String md5Key,
      ) async {
    final Directory _cacheImagesDirectory = Directory(
        join((await getTemporaryDirectory()).path, cacheImageFolderName));
    //exist, try to find cache image file
    if (_cacheImagesDirectory.existsSync()) {
      final File cacheFlie = File(join(_cacheImagesDirectory.path, md5Key));
      if (cacheFlie.existsSync()) {
        return await cacheFlie.readAsBytes();
      }
    }
    //create folder
    else {
      await _cacheImagesDirectory.create();
    }
    //load from network
    final Uint8List data = await _loadNetwork(
      key,
    );
    if (data != null) {
      //cache image file
      await File(join(_cacheImagesDirectory.path, md5Key)).writeAsBytes(data);
      return data;
    }

    return null;
  }


  @override
  bool operator ==(dynamic other) {
    if (other.runtimeType != runtimeType) return false;
    final NetworkImageAsyncProvider typedOther = other;
    return url == typedOther.url && scale == typedOther.scale;
  }

  @override
  int get hashCode => hashValues(url, scale);

  @override
  String toString() => '$runtimeType("$url", scale: $scale)';
}